import { KEYCLOAK_DEVICE_AUTH_URL, KEYCLOAK_TOKEN_CREATE_URL, KEYCLOAK_USER_INFO_URL } from '../config';
import BriefingView from '../view/briefingView';
import QuickSetupView from '../view/quickSetupView';
import { CommandHandler, openUrlInBrowserCommand } from './commandHandler';
import ExtensionStore from './extensionStore';
import KeycloakOAuth2DeviceFlowConnection from './keycloakOAuth2DeviceFlowConnection';
import ApiService from './api.service';
import { GiteaPublicProperties } from '../model/giteaPublicProperties.model';
import { Mission } from './mission/models/mission';
import { MissionDevContainer } from './mission/missionDevContainer';
import { extensionLog as log } from '../recorder/utils/log';
import { commands, ExtensionContext, Uri, window } from 'vscode';
import { createSshKeyFiles } from './sshKeyManager';
import { emptyDirectories, removeFilesOrDirectories } from '../recorder/utils/workdir';
import { missionWorkdir, userSshKeyFolderPath } from './config';
import { hasStatusNumber } from './utils/typeguards';
import { User } from './mission/models/userChallenge';

export default class Controller {
  private static instance: Controller;
  public connection: KeycloakOAuth2DeviceFlowConnection;
  private commandHandler: CommandHandler;
  private briefingView: BriefingView;
  private quickSetupView: QuickSetupView;
  private extensionStore: ExtensionStore;
  private _apiService: ApiService;

  private constructor(context: ExtensionContext) {
    this.extensionStore = ExtensionStore.getInstance(context);
    this.briefingView = new BriefingView();
    this.quickSetupView = new QuickSetupView(context.extensionUri);
    this.commandHandler = new CommandHandler(this);
    this.connection = new KeycloakOAuth2DeviceFlowConnection(
      KEYCLOAK_DEVICE_AUTH_URL,
      KEYCLOAK_TOKEN_CREATE_URL,
      KEYCLOAK_USER_INFO_URL,
    );

    this._apiService = new ApiService(this.connection, this.extensionStore, this);

    this.init();
  }

  public static getInstance(context: ExtensionContext): Controller {
    if (!Controller.instance) {
      Controller.instance = new Controller(context);
    }
    return Controller.instance;
  }

  public get apiService(): ApiService {
    return this._apiService;
  }

  private async init() {
    const that = this;
    window.registerUriHandler({
      handleUri(uri: Uri) {
        const queryParams = new URLSearchParams(uri.query);
        const action = queryParams.get('action');
        // TODO: Should we follow eslint no-case-declarations ?
        const missionId = queryParams.get('missionId');
        const missionVersion = queryParams.get('missionVersion');
        const userId = queryParams.get('userId') ?? undefined;
        log('Opening link', uri);

        switch (action) {
          case 'open-mission':
            if (!missionId || !missionVersion) {
              window.showErrorMessage('Identifiant ou version de la mission incorrect');
            } else {
              that.launchMission(missionId, missionVersion, userId);
            }

            break;

          default:
            window.showInformationMessage('Aucune action trouvée!');
        }
      },
    });

    this.quickSetupView.isAlreadyConnected = (await this.extensionStore.getAccessToken()) !== undefined;
  }

  public async disconnect() {
    await removeFilesOrDirectories(`${userSshKeyFolderPath}/id_rsa`, `${userSshKeyFolderPath}/id_rsa.pub`);
    await emptyDirectories(missionWorkdir);
    await this.extensionStore.clear();
    this.quickSetupView.isAlreadyConnected = false;
  }

  public async createSshKeyPair() {
    const { publicKey, privateKey } = await this.apiService.getUserSshKey();
    await createSshKeyFiles(publicKey, privateKey);
  }

  public async authenticate() {
    await this.connection.registerDevice();
    const tokens = await this.connection.getToken({ openLink: Controller.openBrowserWithUrl });
    await this.extensionStore.setAccessToken(tokens.accessToken);
    await this.extensionStore.setRefreshToken(tokens.refreshToken);
    await this.createSshKeyPair();
    this.quickSetupView.isAlreadyConnected = true;
  }

  public static openBrowserWithUrl(url: string) {
    commands.executeCommand(openUrlInBrowserCommand.cmd, Uri.parse(url));
  }

  public async launchMission(missionId: string, missionVersion: string, userId?: string) {
    window.showInformationMessage(`vous lancez la mission ${missionId}`);

    const hadBeenConnected = (await this.extensionStore.getAccessToken()) !== undefined;

    if (!hadBeenConnected) {
      await this.authenticate();
      window.showInformationMessage('Connexion validée');
    }

    const mission = new Mission(missionId, missionVersion);

    const currentUser = await this.apiService.getCurrentUser();

    const user: User = userId ? await this.apiService.getUser(userId) : currentUser;

    if (!!userId && userId !== currentUser.id) {
      try {
        await this.apiService.grantAccessToRepository(user.id, mission.id);
      } catch (e) {
        if (hasStatusNumber(e)) {
          if (e.status === 403) {
            window.showErrorMessage(`Vous n'avez pas accès à la mission: ${missionId}`);
            return;
          }
        } else {
          throw e;
        }
      }
    }

    const giteaPublicProperties: GiteaPublicProperties = await this.apiService.getGiteaPublicProperties();

    const missionDevcontainer = new MissionDevContainer(user, currentUser, mission, giteaPublicProperties);

    try {
      await missionDevcontainer.open();
    } catch {
      window.showErrorMessage('Impossible de lancer la mission');
    }
  }
}
