import { ExtensionContext, SecretStorage } from 'vscode';
import { extensionLog as log } from '../recorder/utils/log';

export default class ExtensionStore {
  private static instance: ExtensionStore;
  private secretStorage: SecretStorage;

  private constructor(context: ExtensionContext) {
    this.secretStorage = context.secrets;
  }

  public async clear() {
    if (await this.secretStorage.get(SecretStoreKey.AccessTokenKey))
      this.secretStorage.delete(SecretStoreKey.AccessTokenKey);
    if (await this.secretStorage.get(SecretStoreKey.RefreshTokenKey))
      this.secretStorage.delete(SecretStoreKey.RefreshTokenKey);
  }

  public static getInstance(context?: ExtensionContext): ExtensionStore {
    if (!ExtensionStore.instance) {
      if (!context) throw new Error('ExtensionStore should be initiate with a storage first time');
      ExtensionStore.instance = new ExtensionStore(context);
    }

    return ExtensionStore.instance;
  }

  public getAccessToken(): Thenable<string | undefined> {
    return this.readSecret(SecretStoreKey.AccessTokenKey);
  }

  public getRefreshToken(): Thenable<string | undefined> {
    return this.readSecret(SecretStoreKey.RefreshTokenKey);
  }

  public setAccessToken(accessToken: string): Thenable<void> {
    if (!accessToken) {
      log('Attempt to store undefined access token');
      return Promise.resolve();
    }
    return this.storeSecret(SecretStoreKey.AccessTokenKey, accessToken);
  }

  public setRefreshToken(refreshToken: string): Thenable<void> {
    if (!refreshToken) {
      log('Attempt to store undefined refresh token');
      return Promise.resolve();
    }
    return this.storeSecret(SecretStoreKey.RefreshTokenKey, refreshToken);
  }

  private readSecret(key: SecretStoreKey): Thenable<string | undefined> {
    return this.secretStorage.get(key);
  }

  private storeSecret(key: SecretStoreKey, value: string): Thenable<void> {
    return this.secretStorage.store(key, value);
  }
}

enum SecretStoreKey {
  AccessTokenKey = 'access-token-key',
  RefreshTokenKey = 'refresh-token-key',
}
