import { missionWorkdir, userSshKeyFolderPath } from '../config';
import { Base, DockerfileSpecific, LifecycleScripts, VSCodespecific } from './models/devContainer';
import { mkdir, writeFile } from 'fs/promises';
import { Mission } from './models/mission';
import { GiteaPublicProperties } from '../../model/giteaPublicProperties.model';
import { commands, Uri } from 'vscode';
import { existsSync, copyFileSync } from 'fs';
import { join } from 'path';

import assert = require('assert');
import UserChallenge, { User } from './models/userChallenge';
import { REGISTRY_MISSION_URL } from '../../config';

const remoteUserHomeDir = '/home/deadlock';
const remoteMissionDir = `${remoteUserHomeDir}/mission/`;
const remoteGiteaWorkDir = '/workdir';
const defaultSettings = {
  'terminal.integrated.defaultProfile.linux': 'bash',
  'terminal.integrated.profiles.linux': {
    bash: {
      path: '/bin/bash',
    },
  },
};

export class MissionDevContainer {
  private isInit = false;
  private readonly mounts: string[] = [];
  private readonly extentionPath: string;

  private readonly dirs: { missionWorkdir: string; config: string; devcontainer: string; mounted: string };

  constructor(
    private readonly user: User,
    private readonly currentUser: User,
    private readonly mission: Mission,
    private readonly giteaProperties: GiteaPublicProperties,
  ) {
    let prefix = missionWorkdir;
    if (this.isReviewingStudent()) {
      prefix += `/students/${this.user.details.lastName}_${this.user.details.firstName}_${this.user.id}`;
    }
    this.dirs = {
      missionWorkdir: `${prefix}/${this.mission.id}`,
      config: `${prefix}/${this.mission.id}/.config`,
      devcontainer: `${prefix}/${this.mission.id}/.devcontainer`,
      mounted: `${prefix}/${this.mission.id}/mounted`,
    };
    if (process.env.DL_MOUNT_EXTENSION === 'true') {
      this.extentionPath = '/injected-extension/extension.vsix';
    } else {
      this.extentionPath = 'Deadlock.deadlock-coding';
    }
  }

  async open() {
    if (!this.isInit) {
      await this.init();
    }
    assert(existsSync(this.dirs.missionWorkdir));
    await commands.executeCommand('remote-containers.openFolder', Uri.file(this.dirs.missionWorkdir));
  }

  private async init() {
    if (this.isInit) {
      throw new Error(`${MissionDevContainer.constructor.name} Already initialized`);
    }
    this.isInit = true;
    await this.createDirectories(...Object.values(this.dirs));
    await this.createDevContainerFile();
    await this.createUserChallengeJsonFile();
  }

  private isReviewingStudent() {
    return this.currentUser.id !== this.user.id;
  }

  private createUserChallengeJsonFile() {
    return writeFile(
      `${this.dirs.config}/user-challenge.json`,
      (() => {
        const userChallengeJson: UserChallenge = {
          giteaHost: this.giteaProperties.sshHost,
          giteaSshPort: this.giteaProperties.sshPort,
          username: this.user.id.split('-').join(''),
          email: `${this.user.id.split('-').join('')}@deadlock.io`,
          missionId: this.mission.id,
          missionVersion: this.mission.version,
          remoteGitUsername: this.user.id.split('-').join(''),
          currentUserDetails: this.currentUser.details,
          currentUserId: this.currentUser.id,
          remoteUserDetails: this.user.details,
        };
        return JSON.stringify(userChallengeJson, null, 2);
      })(),
    );
  }

  private addInMountRCs(tempFolder: string, ...rcs: string[]): void {
    if (process.env.HOME === undefined) {
      throw new Error('HOME environment variable is not defined');
    }
    for (const rc of rcs) {
      if (existsSync(rc)) {
        const rcPath = join(process.env.HOME, rc);
        const tempRC = join(tempFolder, rc);
        const mountRC = join(remoteUserHomeDir, rc);
        copyFileSync(rcPath, tempRC);
        this.mounts.push(`source=${tempRC},target=${mountRC},type=bind`);
      }
    }
  }

  private setupMounts(): void {
    this.mounts.push(
      `source=${userSshKeyFolderPath},target=/tmp/.ssh,type=bind,consistency=cached,readonly`,
      `source=${this.dirs.config},target=/home/config/,type=bind,consistency=cached,readonly`,
      'source=/etc/hosts,target=/etc/hosts,type=bind,consistency=cached,readonly',
    );
    if (process.env.DL_MOUNT_EXTENSION === 'true') {
      this.mounts.push(`source=${process.env.EXTENSION_PATH},target=${this.extentionPath},type=bind`);
    }
    this.addInMountRCs(this.dirs.missionWorkdir, '.bashrc', '.zshrc');
  }

  private createDevContainerFile(options?: Partial<DockerfileSpecific & Base & VSCodespecific & LifecycleScripts>) {
    this.setupMounts();

    return writeFile(
      `${this.dirs.devcontainer}/devcontainer.json`,
      (() => {
        const devcontainer: Partial<DockerfileSpecific & Base & VSCodespecific & LifecycleScripts> = {
          name: `deadlock-${this.mission.id}`,
          image: `${REGISTRY_MISSION_URL}/${this.mission.id}:${this.mission.version}`,
          containerEnv: {
            WORKDIR: `${remoteMissionDir}`,
          },
          extensions: [this.extentionPath],
          remoteUser: 'deadlock',
          mounts: this.mounts,
          userEnvProbe: 'interactiveShell',
          settings: defaultSettings,
          overrideCommand: true,
          shutdownAction: 'stopContainer',
          workspaceMount: `source=${this.dirs.mounted},target=${remoteMissionDir},type=bind`,
          workspaceFolder: `${remoteMissionDir}`,
          onCreateCommand: `cp -R ${remoteGiteaWorkDir}/* ${remoteMissionDir} && sudo bash /start.desktop.sh`,
          runArgs: ['--privileged'],
          ...options,
        };
        return JSON.stringify(devcontainer, null, 2);
      })(),
    );
  }

  private createDirectories(...directoryPaths: string[]) {
    return Promise.all(directoryPaths.map((directoryPath) => mkdir(directoryPath, { recursive: true })));
  }
}
