import { Branch } from '../../core/gitMission';
import GitMission from '../../core/gitMission';
import { DEADLOCK_WORKDIR_PATH, PROJECT_DEADLOCK_DESKTOP_PATH, PROJECT_SRC_PATH } from '../../core/config';
import { format } from 'date-fns';
import { execSync } from 'child_process';
import { existsSync, promises } from 'fs';
import { join } from 'path';
import { GitError } from 'simple-git';
import { queue } from 'async';
import { extensionWarn, recorderError as error, recorderLog as log } from './log';
import { clearFilesExceptGit, copyGitUserFiles } from './workdir';
import { copyFile } from 'fs/promises';

const { readFile } = promises;

const gitQueue = queue(async (task: CallableFunction) => {
  await task();
}, 1);

export enum CommitFrom {
  Run = 'Run',
  Auto = 'Auto',
  HttpServer = 'HttpServer',
}

export async function commitAndPushOnQueue(gitMission: GitMission, from: CommitFrom) {
  if (from === CommitFrom.Run) {
    gitQueue.push(async () => {
      await mergeMaster(gitMission);
    });
  } else if (gitQueue.length() == 0) {
    gitQueue.push(async () => {
      await commitAndPushCode(gitMission);
    });
  }
}

async function commitAndPushCode(gitMission: GitMission) {
  try {
    await clearFilesExceptGit(PROJECT_SRC_PATH);

    copyGitUserFiles(PROJECT_DEADLOCK_DESKTOP_PATH, PROJECT_SRC_PATH, gitMission.author);
    execSync(
      `rsync -r --exclude .git --exclude npm --exclude target ${PROJECT_DEADLOCK_DESKTOP_PATH}/* ${PROJECT_SRC_PATH}`,
    );
    if (existsSync(join(DEADLOCK_WORKDIR_PATH, '.gitignore'))) {
      await copyFile(join(DEADLOCK_WORKDIR_PATH, '.gitignore'), join(PROJECT_SRC_PATH, '.gitignore'));
    }

    const currentDate = format(new Date(), "HH'H'mm'_'dd/LL/y");
    await gitMission.checkout(Branch.LIVE);
    await gitMission.addAll();
    await gitMission.commit(currentDate);
    await gitMission.push();
    log('Commit & push done.');
  } catch (e) {
    if (e instanceof GitError) {
      error(`[${e.task?.commands}] cannot commitAndPush`);
      error(e.message);
      throw new Error(e.stack);
    } else {
      console.error(`[DEADLOCK-EXTENSION] - Error not handled: ${e}`);
      throw e;
    }
  }
}

async function mergeMaster(gitMission: GitMission) {
  try {
    log('Merging live to master');

    const currentDate = format(new Date(), "HH'H'mm'_'dd/LL/y");

    await gitMission.checkout(Branch.DEFAULT);
    await gitMission.merge(['--squash', '-X', 'theirs', Branch.LIVE]);
    await gitMission.commit(currentDate);
    await gitMission.push();
    log('Merge to master done');
    await gitMission.checkout(Branch.LIVE);
  } catch (e) {
    if (e instanceof GitError) {
      error(`[${e.task?.commands}] cannot commitAndPush`);
      error(e.message);
      throw new Error(e.stack);
    } else {
      console.error(`[DEADLOCK-EXTENSION] - Error not handled: ${e}`);
      throw e;
    }
  }
}

export async function getIgnorePatternFromIgnoreFile(ignoreFile: string): Promise<string[]> {
  if (!existsSync(ignoreFile)) {
    extensionWarn(`${ignoreFile} not found`);
    return ['.git'];
  }

  const fileContent = await readFile(ignoreFile, { encoding: 'utf-8' });
  const lines = fileContent.split(/\s*\r?\n\s*/);
  const filesPatterns = lines
    .filter((line) => line.trimLeft().startsWith('#') || !!line.trim())
    .map((line_1) => `**/${line_1}`);
  return [...filesPatterns, '.git'];
}
