import { exec as execCallback, execSync } from 'child_process';
import { copyFileSync, existsSync, lstatSync, PathLike, readdirSync, renameSync } from 'fs';
import { join } from 'path';
import { log } from './log';
import { promisify } from 'util';
import { mkdir, unlink, rm } from 'fs/promises';

const exec = promisify(execCallback);
const PREFIX = '[DEADLOCK-RECORDER]';

export async function copyProjectSources(
  srcPath: string,
  userMissionWorkdirPath: string,
  excludes: Array<string> = [],
) {
  const excludeCmd = excludes.map((current) => `--exclude ${current}`).join(' ');
  return await exec(
    `sudo rsync -a ${srcPath}/ ${userMissionWorkdirPath} ${excludeCmd} && sudo chown -R deadlock:deadlock /home/deadlock`,
  );
}

export async function clearFilesExceptGit(path: string) {
  readdirSync(path).forEach(async (file) => {
    if (file !== '.git') {
      const curPath = join(path, file);
      if (lstatSync(curPath).isDirectory()) {
        await rm(curPath, { recursive: true });
      } else {
        await unlink(curPath);
      }
    }
  });
}

function renameIfExistsSync(srcPath: PathLike, destPath: PathLike) {
  if (existsSync(srcPath)) {
    renameSync(srcPath, destPath);
  } else {
    log(PREFIX, `Renaming : No ${srcPath} found`);
  }
}

function copyFileIfExistsSync(srcPath: PathLike, destPath: PathLike) {
  if (existsSync(srcPath)) {
    copyFileSync(srcPath, destPath);
  } else {
    log(PREFIX, `Copying file: No ${srcPath} found`);
  }
}

function copyFolderIfExistsSync(srcPath: PathLike, destPath: PathLike) {
  if (existsSync(srcPath)) {
    execSync(`rsync -r ${srcPath}/* ${destPath}`);
  } else {
    log(PREFIX, `Copying folder: No ${srcPath} found`);
  }
}

/**
 *
 * Rename temporary git files back to actual git files.
 *
 * @param path Path of the folder containing user's temporary git project files.
 * @param userId Identifier used to create the temporary files name
 */
export function renameTempToUserGitFiles(path: string, userId: string) {
  renameIfExistsSync(join(path, `user-git-${userId}`), join(path, '.git'));
  renameIfExistsSync(join(path, `user-gitignore-${userId}`), join(path, '.gitignore'));
}

/**
 *
 * Copy user's git project files and paste them with renamed filenames so that
 * they can be saved by the recorder.
 *
 * @param srcPath Path of the folder containing user's git project files to be saved.
 * @param destPath Path where the user's git project files should be pasted.
 * @param userId Identifier used to create the temporary files name
 */
export function copyGitUserFiles(srcPath: string, destPath: string, userId: string) {
  copyFolderIfExistsSync(join(srcPath, '.git'), join(destPath, `user-git-${userId}`));
  copyFileIfExistsSync(join(srcPath, '.gitignore'), join(destPath, `user-gitignore-${userId}`));
}

export async function removeFilesOrDirectories(...paths: string[]) {
  paths.filter(existsSync).forEach(async (path) => {
    await rm(path, { recursive: true });
  });
}

export async function emptyDirectories(...directoryPaths: string[]) {
  directoryPaths.forEach(async (directoryPath) => {
    await rm(directoryPath, { recursive: true });
    await mkdir(directoryPath);
  });
}
