import { commands, Uri } from 'vscode';
import { authenticateCommand, disconnectCommand } from '../core/commandHandler';
import ExtensionStore from '../core/extensionStore';
import { extensionLog as log } from '../recorder/utils/log';
import { openQuickSetupCommand } from '../theia/command';
import { WebviewBase } from './webviewBase';

export const quickSetupId = 'quickSetup';

export default class QuickSetupView extends WebviewBase {
  private extensionUri: Uri;
  private extensionStore: ExtensionStore;
  private _isAlreadyConnected: boolean;

  constructor(extensionUri: Uri) {
    super(quickSetupId, 'QuickSetup', openQuickSetupCommand);
    this.extensionUri = extensionUri;
    this.extensionStore = ExtensionStore.getInstance();
    this._isAlreadyConnected = false;
  }

  set isAlreadyConnected(newVal: boolean) {
    if (this._isAlreadyConnected !== newVal) {
      this._isAlreadyConnected = newVal;
      this.reload();
    }
  }

  render(): string {
    return `
         <head>
            ${this.renderHeaderHtml()}
         </head>
         <body>
            ${this.renderHtmlBody()}
         </body>
      `;
  }

  renderHtmlBody() {
    return `
         <h1>Quick Setup</h1>
         <div class="deadlock-getting-started-card-container">

            ${this.renderCardHtml(
              'Connexion à Deadlock',
              "Tu as besoin d'être connecté à Deadlock pour continuer.",
              this._isAlreadyConnected
                ? {
                    name: 'Déconnexion',
                    onClickFunctionName: 'disconnectUserAction',
                  }
                : {
                    name: 'Connexion',
                    onClickFunctionName: 'openAuthenticationPageAction',
                  },
              this._isAlreadyConnected,
            )}
         </div>
         `;
  }

  renderCardHtml(
    title: string,
    description: string,
    button: { name: string; onClickFunctionName: string },
    isChecked: boolean,
    isDisabled?: boolean,
    callbackArgs?: string,
  ) {
    return `
            <div class="deadlock-getting-started-card">
               <vscode-checkbox ${isChecked ? 'checked' : ''} readonly> </vscode-checkbox>
               <div class="card-body">
                  <div class="card-title">
                     ${title}
                  </div>
                  <div class="card-description">
                     ${description}
                  </div>
                  <vscode-button ${isDisabled ? 'disabled' : ''} onclick="${
      button.onClickFunctionName
    }(${callbackArgs})">${button.name}</vscode-button>
               </div>
            </div>
         `;
  }

  renderHeaderHtml() {
    const toolkitUri = this.getExternalRessourcePath(this.extensionUri, [
      'node_modules',
      '@vscode',
      'webview-ui-toolkit',
      'dist',
      'toolkit.js',
    ]);

    const customStyle = this.getExternalRessourcePath(this.extensionUri, [
      'resources',
      'styles',
      'gettingStartedView.css',
    ]);

    const customScript = this.getExternalRessourcePath(this.extensionUri, ['resources', 'js', 'gettingStartedView.js']);

    return `
         <meta charset="UTF-8">
         <script type="module" src="${toolkitUri}"></script>
         <script type="text/javascript" src="${customScript}"></script>
         <link href="${customStyle}" rel="stylesheet" />
      `;
  }

  onMessageReceive(message: any): void {
    switch (message.command) {
      case 'openAuthenticationPageAction':
        commands.executeCommand(authenticateCommand.cmd);
        return;
      case 'disconnectUserAction':
        commands.executeCommand(disconnectCommand.cmd);
        return;
    }
  }

  load(): void {
    log('Loading GettingStartedView');
  }
}
