import { randomBytes } from 'crypto';
import { existsSync, readdirSync, readFileSync } from 'fs';
import { join } from 'path';
import { ExtensionContext, Webview, WebviewView, WebviewViewProvider, window } from 'vscode';
import { missionWorkdir } from '../core/config';
import Controller from '../core/controller';
import UserChallenge from '../core/mission/models/userChallenge';
import { extensionWarn } from '../recorder/utils/log';
import { getUri } from './webviewBase';

export default class StartedMissionsView implements WebviewViewProvider {
  private readonly controller: Controller;
  constructor(private context: ExtensionContext) {
    this.controller = Controller.getInstance(context);
  }

  resolveWebviewView(webviewView: WebviewView): void | Thenable<void> {
    webviewView.webview.options = {
      enableScripts: true,

      localResourceRoots: [this.context.extensionUri],
    };
    webviewView.webview.html = this._getHtmlForWebview(webviewView.webview);
    webviewView.webview.onDidReceiveMessage(this.onMessageReceive.bind(this));
  }

  onMessageReceive(message: any): void {
    switch (message.command) {
      case 'openMission': {
        window.showInformationMessage('click received: ' + message.mission);
        const path = join(missionWorkdir, message.mission, '.config', 'user-challenge.json');
        const userChallenge: UserChallenge = JSON.parse(readFileSync(path, 'utf8'));
        this.controller.launchMission(userChallenge.missionId, userChallenge.missionVersion);
        return;
      }
    }
  }

  private _getHtmlForWebview(webview: Webview) {
    const toolkitUri = getUri(webview, this.context.extensionUri, [
      'node_modules',
      '@vscode',
      'webview-ui-toolkit',
      'dist',
      'toolkit.js',
    ]);

    const css = getUri(webview, this.context.extensionUri, ['resources', 'styles', 'startedMissionsView.css']);

    const js = getUri(webview, this.context.extensionUri, ['resources', 'js', 'startedMissionsView.js']);

    // find all folders in mission directory
    let missionsHtml = '';
    existsSync(missionWorkdir) &&
      readdirSync(missionWorkdir).forEach((mission) => {
        if (existsSync(join(missionWorkdir, mission, '.config', 'user-challenge.json'))) {
          try {
            const userChallenge: UserChallenge = JSON.parse(
              readFileSync(join(missionWorkdir, mission, '.config', 'user-challenge.json'), 'utf8'),
            );
            missionsHtml += `<vscode-button onclick="openMission('${userChallenge.missionId}')" class="item" appearance="primary">${userChallenge.missionId}</vscode-button>`;
          } catch (e) {
            extensionWarn(`Failed to read user-challenge.json for mission ${mission}`);
          }
        } else {
          extensionWarn(`Folder '${mission}' has no user-challenge.json file so it is ignored.`);
        }
      });

    if (!missionsHtml) {
      missionsHtml = '<div class="item">Aucune mission n\'a été démarrée</div>';
    }

    return `<!DOCTYPE html>
    <html lang="en">
    <head>
        <meta charset="UTF-8">
        <script type="module" src="${toolkitUri}"></script>
        <script nonce="${randomBytes(16).toString('base64')}" src="${js}"></script>
        <link rel="stylesheet" type="text/css" href="${css}">
        </head>
    <body>
      <div>
        ${missionsHtml}
      </div>
    </body>
    </html>`;
  }
}
