'use strict';
import {
  commands,
  Disposable,
  Uri,
  ViewColumn,
  Webview,
  WebviewPanel,
  WebviewPanelOnDidChangeViewStateEvent,
  window,
} from 'vscode';
import { Command } from '../theia/command';

const emptyCommands: Disposable[] = [
  {
    dispose: function () {
      /* noop */
    },
  },
];

export function getUri(webview: Webview, extensionUri: Uri, pathList: string[]) {
  return webview.asWebviewUri(Uri.joinPath(extensionUri, ...pathList));
}

export abstract class WebviewBase implements Disposable {
  protected disposable: Disposable;
  private disposablePanel: Disposable | undefined;
  protected panel: WebviewPanel | undefined;

  constructor(private id: string, private title: string, command: Command, private readonly _column?: ViewColumn) {
    this.disposable = Disposable.from(commands.registerCommand(command.cmd, this.onShowCommand, this));
    this.load();
  }

  getExternalRessourcePath(extensionUri: Uri, pathList: string[]) {
    return getUri(this.panel!.webview, extensionUri, pathList);
  }

  registerCommands(): Disposable[] {
    return emptyCommands;
  }

  renderHead?(): string | Promise<string>;
  renderBody?(): string | Promise<string>;
  renderEndOfBody?(): string | Promise<string>;

  dispose() {
    this.disposable.dispose();
    this.disposablePanel?.dispose();
  }

  protected onShowCommand() {
    void this.show();
  }

  private onPanelDisposed() {
    this.disposablePanel?.dispose();
    this.panel = undefined;
  }

  private onViewStateChanged(e: WebviewPanelOnDidChangeViewStateEvent) {
    console.log(
      `Webview(${this.id}).onViewStateChanged`,
      `active=${e.webviewPanel.active}, visible=${e.webviewPanel.visible}`,
    );
  }

  get visible() {
    return this.panel?.visible ?? false;
  }

  hide() {
    this.panel?.dispose();
  }

  setTitle(title: string) {
    if (this.panel == null) return;

    this.panel.title = title;
  }

  onMessageReceive(message) {
    switch (message.command) {
      case 'alert':
        window.showErrorMessage(message.text);
        return;
    }
  }

  async show(): Promise<void> {
    if (this.panel == null) {
      this.panel = window.createWebviewPanel(this.id, this.title, ViewColumn.Active, {
        retainContextWhenHidden: true,
        enableFindWidget: true,
        enableCommandUris: true,
        enableScripts: true,
      });

      this.disposablePanel = Disposable.from(
        this.panel,
        this.panel.onDidDispose(this.onPanelDisposed, this),
        this.panel.onDidChangeViewState(this.onViewStateChanged, this),
        this.panel.webview.onDidReceiveMessage(this.onMessageReceive, this),
        ...this.registerCommands(),
      );

      this.panel.webview.html = await this.getHtml();
    } else {
      const html = await this.getHtml();

      // Reset the html to get the webview to reload
      this.panel.webview.html = '';
      this.panel.webview.html = html;

      this.panel.reveal(this.panel.viewColumn ?? ViewColumn.Active, false);
    }
  }

  async reload() {
    if (this.panel) {
      const html = await this.getHtml();

      // Reset the html to get the webview to reload
      this.panel.webview.html = '';
      this.panel.webview.html = html;
    }
  }

  /**
   * Must return the content displayed within the webViewPanel
   */
  abstract render(): string;
  /**
   * Method called at the end of first render
   */
  abstract load(): void;

  private async getHtml(): Promise<string> {
    return this.render();
  }
}
